%StaticCameraMovingObjectDemo1
close all
clear all

imgFilename = [];
imgFilename{end+1} = 'Moose1.png';
imgFilename{end+1} = 'Moose2.png';
imgFilename{end+1} = 'Moose3.png';

numRuns = length(imgFilename);

for runLoop = 1:numRuns,
   
    imCell{runLoop} = imread(imgFilename{runLoop});
    imCell{runLoop}  = imCell{runLoop} (:,:,1); 
    imCell{runLoop}  = im2double(imCell{runLoop} );
    EvalPrint('size(imCell{runLoop} )')
    polyBoundaryVecCell{runLoop} = ExtractBoundaries( imCell{runLoop}  );    
    polyBoundaryVecCell{runLoop} = OrderMostConsistentEpipoles(  polyBoundaryVecCell{runLoop} );    
end

[viewVec, vecInit] = StaticMirrorObjMovingCamFn( ...
    {polyBoundaryVecCell{1}}, 'draw', 1, 'doFinalLm', 1, 'doLm', 0, 'mfe', 150, 'tol', 1e-13 );

resInit = StaticMirrorCameraMovingObjErr( vecInit, ones(size(vecInit)), polyBoundaryVecCell );
EvalPrint('rms(resInit)')

tol = 1e-4;
mfe  = 100;

options=optimset( 'LevenbergMarquardt','on', 'MaxFunEvals',mfe,  'TolFun',tol,...
    'LargeScale','on', 'TolX',tol, 'Display', 'iter');

vecOpt = lsqnonlin('StaticMirrorCameraMovingObjErr', ones(size(vecInit)), [], [], options, vecInit, polyBoundaryVecCell );            
vecOpt = vecOpt.*vecInit;

[resOpt, theCameraVec] = StaticMirrorCameraMovingObjErr( vecOpt, ones(size(vecOpt)), polyBoundaryVecCell );
EvalPrint('rms(resOpt)')

quatVec = SlerpedQuaternionGridSet(4); %orientation starting points
vecQuatTranInit = [];
for runLoop = 1:numRuns
    viewVecCell{runLoop} = FormViewVec( polyBoundaryVecCell{runLoop}, theCameraVec );
    if runLoop > 1
        [vecPerspOpt, rmsErr, rmsVec] = VemhEtAlign( viewVecCell{1}, viewVecCell{runLoop}  );rmsErr
        EvalPrint('rmsVec')    
        vecQuatTranInit = [vecQuatTranInit; vecPerspOpt];
    end
end

mergedViewVec = viewVecCell{1};
for runLoop = 2:numRuns
    mergedViewVec = MergeViewVec( mergedViewVec, viewVecCell{runLoop}, vecQuatTranInit( (1:7) + 7*(runLoop-2) ) );
end
EvalPrint('rms(PixelEtErrMex(mergedViewVec))')  
EvalPrint('MeanCaliperDiameter(mergedViewVec)')  

[pMesh, kMesh] = VhMarchTetMirror( mergedViewVec, imCell, 'draw', 1, 'cellsAcross', 100);

vrmlFilename = sprintf('VisualHullRuns%d.wrl',  numRuns );
fid = fopen( vrmlFilename, 'w' );
fprintf(fid, '#VRML V2.0 utf8\n');
fprintf(fid, 'Background {skyColor [0.7 0.7 0.7]}\n');
cameraPosition = [0;0;5];
cameraTarget = ViewVecCentrePoint( mergedViewVec );
AddVrmlViewpoint( fid, cameraPosition, cameraTarget, 'description', 'visual hull', 'fieldOfView', 0.2 );
AddVrmlIndexedFaceSet( fid, pMesh, kMesh, 'objNameStr', 'DoubleMirror');
fclose(fid);  
